unit CheckRxDBGrid;
// v1.1 03.08.2001 M.Peter - mpeter@web.de
//
// create Interbase-Domain with:
// CREATE DOMAIN "BOOLEAN" AS CHAR(1) DEFAULT 'F' CHECK (VALUE IN ('F','T')) NOT NULL;


interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Grids, DB, DBGrids, CheckDBGrid , RxDBCtrl;

type
  TCheckRxDBGrid = class(TRxDBGrid)
   private
    Editing :Boolean;
    FAllowAppend: Boolean;
    OriginalOptions: TDBGridOptions;
    function ValueMatch(const ValueList, Value: string): Boolean;
    procedure SaveBoolean;
   protected
    function  CreateColumns: TDBGridColumns; override;
   protected
    procedure CellClick(Column: TColumn); override;
    procedure ColEnter; override;
    procedure ColExit; override;
    procedure DoEnter; override;
    Procedure DoExit; override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure DrawColumnCell(const Rect: TRect; DataCol: Integer; Column: TColumn; State: TGridDrawState); override;
   published
    property AllowAppend: Boolean read FAllowAppend write FAllowAppend default True;
  end;

procedure Register;

implementation

function TCheckRxDBGrid.ValueMatch(const ValueList, Value: string): Boolean;
  var Pos: Integer;
  begin
    Result := False;
    Pos := 1;
    while Pos <= Length(ValueList) do
      if AnsiCompareText(ExtractFieldName(ValueList, Pos), Value) = 0 then
      begin
        Result := True;
        Break;
      end;
  end;

function TCheckRxDBGrid.CreateColumns: TDBGridColumns;
  begin
    Result := TDBGridColumns.Create(Self, TCheckColumn);
  end;

procedure TCheckRxDBGrid.SaveBoolean;
  var column :TCheckColumn;
      pos    :integer;
  begin
    column:=TCheckColumn(Columns[SelectedIndex]);
    if not column.ReadOnly then begin
      SelectedField.Dataset.Edit;
      if SelectedField.DataType=ftBoolean then
       SelectedField.AsBoolean:=not SelectedField.AsBoolean
      else begin
        pos:=1;
        if ValueMatch(column.ValueChecked, SelectedField.AsString) then
         SelectedField.AsString:=ExtractFieldName(column.ValueUnchecked,pos)
        else
         SelectedField.AsString:=ExtractFieldName(column.ValueChecked,pos);
      end;
      SelectedField.Dataset.Post;
    end;
    inherited;
  end;

procedure TCheckRxDBGrid.CellClick(Column: TColumn);
  begin
    if TCheckColumn(Column).Check then SaveBoolean;
    inherited;
  end;

procedure TCheckRxDBGrid.ColEnter;
  begin
    if TCheckColumn(Columns[SelectedIndex]).Check then begin
      Editing:=dgEditing in Options;
      Options:=Options-[dgEditing];
    end;
    inherited;
  end;

procedure TCheckRxDBGrid.DoEnter;
  begin
    if TCheckColumn(Columns[SelectedIndex]).Check then begin
      Editing:=dgEditing in Options;
      Options:=Options-[dgEditing];
    end;
    inherited;
  end;

procedure TCheckRxDBGrid.ColExit;
  begin
    if (TCheckColumn(Columns[SelectedIndex]).Check)and Editing then
     Options:=Options+[dgEditing];
    inherited;
  end;

procedure TCheckRxDBGrid.DoExit;
  begin
    if (TCheckColumn(Columns[SelectedIndex]).Check)and Editing then
     Options:=Options+[dgEditing];
    inherited;
  end;

procedure TCheckRxDBGrid.KeyDown(var Key: Word; Shift: TShiftState);
  begin
    inherited;
    if (Key=VK_SPACE)and TCheckColumn(Columns[SelectedIndex]).Check then SaveBoolean;
    if (not FAllowAppend)and(DataLink.DataSet.eof)and(Key=VK_DOWN) then begin
      Key:=0;
      DataLink.DataSet.Cancel;
    end;
  end;

procedure TCheckRxDBGrid.DrawColumnCell(const Rect: TRect; DataCol: Integer; Column: TColumn; State: TGridDrawState);
  const CtrlState :array[Boolean] of Integer = (DFCS_BUTTONCHECK, DFCS_BUTTONCHECK or DFCS_CHECKED);
  begin
    inherited;
    if TCheckColumn(Column).Check then begin
      Canvas.FillRect(Rect);
      DrawFrameControl(Canvas.Handle,Rect,DFC_BUTTON,CtrlState[Column.Field.AsBoolean]);
    end;
 end;

procedure Register;
begin
  RegisterComponents('CheckGrids', [TCheckRxDBGrid]);
end;

end.
