
{*******************************************************}
{                                                       }
{       MiTeC System Information Component              }
{               Disk Detection Part                     }
{           version 6.0 for Delphi 5,6                  }
{                                                       }
{       Copyright  1997,2001 Michal Mutl               }
{                                                       }
{*******************************************************}

{$INCLUDE MITEC_DEF.INC}

unit MSI_Disk;

interface

uses
  SysUtils, Windows, Classes, MiTeC_Routines;

type
  TDisk = class(TPersistent)
  private
    FDisk: TDiskSign;
    FMediaPresent: Boolean;
    FDriveType: TMediaType;
    FSectorsPerCluster: DWORD;
    FBytesPerSector: DWORD;
    FFreeClusters: DWORD;
    FTotalClusters: DWORD;
    FFileFlags: TFileFlags;
    FVolumeLabel: string;
    FSerialNumber: string;
    FFileSystem: string;
    FFreeSpace: int64;
    FCapacity: int64;
    FAvailDisks: string;
    FSerial: dword;
    function GetMediaPresent: Boolean;
  protected
    procedure SetDisk(const Value: TDiskSign);
  public
    procedure GetInfo;
    procedure Report_FileFlags(var AFileFlags :TStringList);
    procedure Report(var sl :TStringList);
    function GetCD :byte;
    property Serial :dword read FSerial {$IFNDEF D6PLUS} write FSerial {$ENDIF} stored false;
  published
    property Drive :TDiskSign read FDisk write SetDisk stored false;
    property AvailableDisks :string read FAvailDisks {$IFNDEF D6PLUS} write FAvailDisks {$ENDIF} stored false;
    property MediaPresent :Boolean read GetMediaPresent {$IFNDEF D6PLUS} write FMediaPresent {$ENDIF} stored false;
    property MediaType :TMediaType read FDriveType {$IFNDEF D6PLUS} write FDriveType {$ENDIF} stored false;
    property FileFlags :TFileFlags read FFileFlags {$IFNDEF D6PLUS} write FFileFlags {$ENDIF} stored false;
    property FileSystem :string read FFileSystem {$IFNDEF D6PLUS} write FFileSystem {$ENDIF} stored false;
    property FreeClusters :DWORD read FFreeClusters {$IFNDEF D6PLUS} write FFreeClusters {$ENDIF} stored false;
    property TotalClusters :DWORD read FTotalClusters {$IFNDEF D6PLUS} write FTotalClusters {$ENDIF} stored false;
    // FreeSpace and Capacity returns good results for Win95 OSR2, Win98, NT and 2000
    // for Win95 there can be bad sizes for drives over 2GB
    property FreeSpace :int64 read FFreeSpace {$IFNDEF D6PLUS} write FFreeSpace {$ENDIF} stored false;
    property Capacity :int64 read FCapacity {$IFNDEF D6PLUS} write FCapacity {$ENDIF} stored false;
    property SerialNumber :string read FSerialNumber {$IFNDEF D6PLUS} write FSerialNumber {$ENDIF} stored false;
    property VolumeLabel :string read FVolumeLabel {$IFNDEF D6PLUS} write FVolumeLabel {$ENDIF} stored false;
    property SectorsPerCluster :DWORD read FSectorsPerCluster {$IFNDEF D6PLUS} write FSectorsPerCluster {$ENDIF} stored false;
    property BytesPerSector :DWORD read FBytesPerSector {$IFNDEF D6PLUS} write FBytesPerSector {$ENDIF} stored false;
  end;

implementation

{ TDisk }

function TDisk.GetCD: byte;
var
  i :integer;
  root :pchar;
begin
  result:=0;
  root:=stralloc(255);
  for i:=1 to length(FAvailDisks) do begin
    strpcopy(root,copy(FAvailDisks,i,1)+':\');
    if getdrivetype(root)=drive_cdrom then begin
      result:=i;
      break;
    end;
  end;
  strdispose(root);
end;

procedure TDisk.Report_FileFlags;
begin
  with AFileFlags do begin
    Add(Format('Case Is Preserved=%d',[integer(fsCaseIsPreserved in FileFlags)]));
    Add(Format('Case Sensitive=%d',[integer(fsCaseSensitive in FileFlags)]));
    Add(Format('Unicode {$ENDIF} stored On Disk=%d',[integer(fsUnicodeStoredOnDisk in FileFlags)]));
    Add(Format('Persistent Acls=%d',[integer(fsPersistentAcls in FileFlags)]));
    Add(Format('File Compression=%d',[integer(fsFileCompression in FileFlags)]));
    Add(Format('Volume Is Compressed=%d',[integer(fsVolumeIsCompressed in FileFlags)]));
    Add(Format('Long Filenames=%d',[integer(fsLongFileNames in FileFlags)]));
    Add(Format('Encrypted File System Support=%d',[integer(fsEncryptedFileSystemSupport in FileFlags)]));
    Add(Format('Object IDs Support=%d',[integer(fsObjectIDsSupport in FileFlags)]));
    Add(Format('Reparse Points Support=%d',[integer(fsReparsePointsSupport in FileFlags)]));
    Add(Format('Sparse Files Support=%d',[integer(fsSparseFilesSupport in FileFlags)]));
    Add(Format('Disk Quotas Support=%d',[integer(fsDiskQuotasSupport in FileFlags)]));
  end;
end;

procedure TDisk.GetInfo;
var
  i,n :integer;
  buf :pchar;
begin
  buf:=stralloc(255);
  n:=GetLogicalDriveStrings(255,buf);
  FAvailDisks:='';
  for i:=0 to n do
    if buf[i]<>#0 then begin
      if (ord(buf[i]) in [$41..$5a]) or (ord(buf[i]) in [$61..$7a]) then
        FAvailDisks:=FAvailDisks+upcase(buf[i])
    end else
      if buf[i+1]=#0 then
        break;
  strdispose(buf);
end;


function TDisk.GetMediaPresent :Boolean;
begin
  Result:=MiTeC_Routines.GetMediaPresent(FDisk);
end;

procedure TDisk.Report(var sl: TStringList);
var
  i :integer;
begin
  with sl do begin
    Add('[Disks]');
    Add(Format('Available=%s',[AvailableDisks]));
    for i:=1 to Length(AvailableDisks) do begin
      Drive:=copy(AvailableDisks,i,1)+':';
      Add(Format('[Disk%s]',[Drive]));
      Add(Format('VolumeLabel=%s',[VolumeLabel]));
      Add(Format('Type=%s',[GetMediaTypeStr(MediaType)]));
      Add(Format('UNC=%s',[ExpandUNCFilename(Drive)]));
      Add(Format('SerialNumber=%s',[SerialNumber]));
      {$IFDEF D4PLUS}
      Add(FormatFloat('Capacity=0,##',Capacity));
      Add(FormatFloat('FreeSpace=0,##',FreeSpace));
      {$ELSE}
      Add(FormatFloat('Capacity=0,##',Capacity.QuadPart));
      Add(FormatFloat('FreeSpace=0,##',FreeSpace.QuadPart));
      {$ENDIF}
      Add(FormatFloat('BytesPerSector=0',BytesPerSector));
      Add(FormatFloat('SectorPerCluster=0',SectorsPerCluster));
      Add(FormatFloat('FreeClusters=0,##',FreeClusters));
      Add(FormatFloat('TotalClusters=0,##',TotalClusters));
      Add(Format('[Disk%s_Flags]',[Drive]));
      Report_FileFlags(sl);
    end;
  end;
end;

procedure TDisk.SetDisk(const Value: TDiskSign);
var
  DI: TDiskInfo;
begin
  FDisk:=Value;
  DI:=GetDiskInfo(Value);
  FDriveType:=DI.MediaType;
  FFileFlags:=DI.FileFlags;
  FCapacity:=DI.Capacity;
  FFreeSpace:=DI.FreeSpace;
  FBytesPerSector:=DI.BytesPerSector;
  FTotalClusters:=DI.TotalClusters;
  FFreeClusters:=DI.FreeClusters;
  FSectorsPerCluster:=DI.SectorsPerCluster;
  FVolumeLabel:=DI.VolumeLabel;
  FFileSystem:=DI.FileSystem;
  FSerialNumber:=DI.SerialNumber;
  FSerial:=DI.Serial;
end;


end.
