
{*******************************************************}
{                                                       }
{       MiTeC System Information Component              }
{               Printer Detection Part                  }
{           version 6.0 for Delphi 5,6                  }
{                                                       }
{       Copyright  1997,2001 Michal Mutl               }
{                                                       }
{*******************************************************}

{$INCLUDE MITEC_DEF.INC}

unit MSI_Printers;

interface

uses
  SysUtils, Windows, Classes;

type
  TPrinters = class(TPersistent)
  private
    FPrinter, FPort: TStringlist;
    FDI: integer;
    function GetPorts: TStrings;
    function GetPrinters: TStrings;
    procedure SetPorts(const Value: TStrings);
    procedure SetPrinters(const Value: TStrings);
  public
    constructor Create;
    destructor Destroy; override;
    procedure GetInfo;
    procedure Report(var sl :TStringList);
  published
    property DefaultIndex: integer read FDI {$IFNDEF D6PLUS} write FDI {$ENDIF} stored False;
    property Names :TStrings read GetPrinters {$IFNDEF D6PLUS} write SetPrinters {$ENDIF} stored false;
    property Ports: TStrings read GetPorts {$IFNDEF D6PLUS} write SetPorts {$ENDIF} stored False;
  end;

implementation

uses Printers, MiTeC_Routines;

{ TPrinters }

constructor TPrinters.Create;
begin
  FPort:=TStringList.Create;
  FPrinter:=TStringList.Create;
end;

destructor TPrinters.Destroy;
begin
  FPrinter.Free;
  FPort.Free;
  inherited;
end;

procedure TPrinters.GetInfo;
var
  i :integer;
  Device, Driver, Port: PChar;
  Mode: THandle;
begin
  Device:=AllocMem(MAX_PATH);
  Driver:=AllocMem(MAX_PATH);
  Port:=AllocMem(MAX_PATH);
  FPrinter.Clear;
  FPort.Clear;
  for i:=0 to Printer.Printers.count-1 do begin
    Printer.PrinterIndex:=i;
    Printer.GetPrinter(Device,Driver,Port,Mode);
    FPrinter.Add(Device);
    FPort.Add(Port);
  end;
  Printer.PrinterIndex:=-1;
  Printer.GetPrinter(Device,Driver,Port,Mode);
  FDI:=FPrinter.IndexOf(Device);
  FreeMem(Device);
  FreeMem(Port);
  FreeMem(Driver);
end;

function TPrinters.GetPorts: TStrings;
begin
  Result:=FPort;
end;

function TPrinters.GetPrinters: TStrings;
begin
  Result:=FPrinter;
end;

procedure TPrinters.Report(var sl: TStringList);
var
  i: integer;
begin
  with sl do begin
    Add('[Printers]');
    Add(Format('Count=%d',[Self.Names.Count]));
    for i:=0 to Self.Names.Count-1 do
      Add(Format('%s on %s=%d',[Self.Names[i],Self.Ports[i],integer(DefaultIndex=i)]));
  end;
end;

procedure TPrinters.SetPorts(const Value: TStrings);
begin

end;

procedure TPrinters.SetPrinters(const Value: TStrings);
begin

end;

end.
